@@ -1,6 +1,6 @@
 -- luatexko.lua
 --
--- Copyright (c) 2013-2017  Dohyun Kim  <nomos at ktug org>
+-- Copyright (c) 2013-2018  Dohyun Kim  <nomos at ktug org>
 --                          Soojin Nam  <jsunam at gmail com>
 --
 -- This work may be distributed and/or modified under the
@@ -13,8 +13,8 @@
 
 luatexbase.provides_module {
   name        = 'luatexko',
-  date        = '2017/02/06',
-  version     = '1.16',
+  date        = '2018/04/06',
+  version     = '1.19',
   description = 'Korean linebreaking and font-switching',
   author      = 'Dohyun Kim, Soojin Nam',
   license     = 'LPPL v1.3+',
@@ -217,6 +217,7 @@ local inhibitxspcode = setmetatable({
   [0x00B0] = 1, -- °
   [0x2032] = 1, -- ′
   [0x2033] = 1, -- ″
+  [0x25CC] = 2, -- dotted circle
   [0x3012] = 2, -- 〒
   [0x301C] = 0, -- 〜
   [0xFE14] = 1, -- ︔
@@ -486,8 +487,7 @@ local function is_cjk_k (c)
   or (c >= 0xFE10  and c <= 0xFE1F )
   or (c >= 0xFE30  and c <= 0xFE4F )
   or (c >= 0xFF00  and c <= 0xFFEF )
-  or (c >= 0x20000 and c <= 0x2A6DF)
-  or (c >= 0x2F800 and c <= 0x2FA1F)
+  or (c >= 0x20000 and c <= 0x2FA1F)
   or  c == 0x00B0  or  c == 0x2015
   --  or  c == 0x2018  or  c == 0x2019
   --  or  c == 0x201C  or  c == 0x201D
@@ -498,8 +498,7 @@ end
 local function is_hanja (c)
   return (c >= 0x3400 and c <= 0x9FFF )
   or (c >= 0xF900 and c <= 0xFAFF )
-  or (c >= 0x20000 and c <= 0x2A6DF)
-  or (c >= 0x2F800 and c <= 0x2FA1F)
+  or (c >= 0x20000 and c <= 0x2FA1F)
 end
 
 local function is_hangul (c)
@@ -1607,38 +1606,43 @@ end
 -- reorder hangul tone marks
 -----------------------------
 local function reorderTM (head)
-  local curr = d_nodetail(head)
+  local curr, todo = head, nil
   while curr do
-    if d_getid(curr) == glyphnode then
+    local nid = d_getid(curr)
+    if nid == glyphnode then
       local uni = d_get_unicode_char(curr)
       if uni == 0x302E or uni == 0x302F then
-        local done = false
-        local unichar = get_font_char(d_getfont(curr), uni)
-        if unichar and unichar.width > 0 then
-          local p = d_getprev(curr)
-          while p and d_getid(p) == glyphnode do
-            local pc = get_cjk_class(d_get_unicode_char(p))
-            if pc == 9 then
-            elseif pc == 7 or pc == 8 then
-              head = d_remove_node(head,curr)
-              d_setprev(curr, nil) -- prev might survive!
-              head, curr = d_insert_before(head,p,curr)
-              done = true
-              break
-            else
-              break
-            end
-            p = d_getprev(p)
+        local tmgl = get_font_char(d_getfont(curr), uni)
+        local tmwidth = tmgl and tmgl.width or 0
+        if todo then
+          if tmwidth > 0 then
+            local tm = curr
+            head, curr = d_remove_node(head, curr)
+            d_setprev(tm, nil) -- prev might survive!
+            head = d_insert_before(head, todo, tm)
+            curr = d_getprev(curr)
+          end
+        elseif get_font_char(d_getfont(curr), 0x25CC) then
+          local u25CC = d_copy_node(curr)
+          d_setchar(u25CC, 0x25CC)
+          if tmwidth > 0 then
+            head, curr = d_insert_after(head, curr, u25CC)
+          else
+            head = d_insert_before(head, curr, u25CC)
           end
         end
-        if not done and get_font_char(d_getfont(curr), 0x25CC) then
-          local u_25CC = d_copy_node(curr)
-          d_setchar(u_25CC, 0x25CC)
-          d_insert_after(head, curr, u_25CC)
-        end
+        todo = nil
+      elseif is_hangul(uni) or is_chosong(uni) or uni == 0x25CC then
+        todo = curr
+      elseif is_jungjongsong(uni) then
+      else
+        todo = nil
       end
+    elseif nid == kernnode then
+    else
+      todo = nil
     end
-    curr = d_getprev(curr)
+    curr = d_getnext(curr)
   end
   return head
 end
