@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# $Id: install-menu-text.pl 44571 2017-06-12 13:23:16Z siepo $
+# $Id: install-menu-text.pl 46802 2018-03-03 21:53:20Z karl $
 # install-menu-txt.pl
 #
 # Copyright 2007-2017 Norbert Preining, Karl Berry
@@ -793,7 +793,7 @@ sub options_menu {
   print <<"EOF";
  <P> use letter size instead of A4 by default: $b_letter
  <E> execution of restricted list of programs: $b_restricted
- <F> create format files:                      $b_fmt
+ <F> create all format files:                  $b_fmt
 EOF
 ;
   if ($vars{'doc_splitting_supported'} and !$opt_in_place) {
